package ru.dbotthepony.kstarbound.world.api

import ru.dbotthepony.kstarbound.RegistryObject
import ru.dbotthepony.kstarbound.defs.tile.LiquidDefinition
import ru.dbotthepony.kstarbound.defs.tile.MaterialModifier
import ru.dbotthepony.kstarbound.defs.tile.TileDefinition
import java.io.DataInputStream

interface IChunkCell {
	// absolute in-world coordinates
	val x: Int
	val y: Int

	val foreground: ITileState
	val background: ITileState
	val liquid: ILiquidState

	var dungeonId: Int
	var biome: Int
	var envBiome: Int
	var isIndestructible: Boolean

	fun read(
		materialAccess: (Int) -> RegistryObject<TileDefinition>?,
		modifierAccess: (Int) -> RegistryObject<MaterialModifier>?,
		liquidAccess: (Int) -> RegistryObject<LiquidDefinition>?,
		stream: DataInputStream
	) {
		foreground.read(materialAccess, modifierAccess, stream)
		background.read(materialAccess, modifierAccess, stream)
		liquid.read(liquidAccess, stream)

		stream.skipNBytes(1) // collisionMap

		dungeonId = stream.readUnsignedShort()
		biome = stream.readUnsignedByte()
		envBiome = stream.readUnsignedByte()
		isIndestructible = stream.readBoolean()

		stream.skipNBytes(1) // unknown
	}

	companion object {
		fun skip(stream: DataInputStream) {
			ITileState.skip(stream)
			ITileState.skip(stream)
			ILiquidState.skip(stream)
			stream.skipNBytes(1 + 2 + 1 + 1 + 1 + 1)
		}
	}
}